@@ -60,14 +60,15 @@ typedef struct {
   int delay_buf_full;       /* Shows buffer situation (important for drain) */
 } priv_t;
 
-static int getopts(sox_effect_t * effp, int n, char * * argv)
+static int getopts(sox_effect_t * effp, int argc, char * * argv)
 {
   priv_t * l = (priv_t *) effp->priv;
   char * s;
   char dummy;     /* To check for extraneous chars. */
   unsigned pairs, i, j, commas;
 
-  if (n < 2 || n > 5)
+  --argc, ++argv;
+  if (argc < 2 || argc > 5)
     return lsx_usage(effp);
 
   /* Start by checking the attack and decay rates */
@@ -95,7 +96,7 @@ static int getopts(sox_effect_t * effp, int n, char * * argv)
     }
   }
 
-  if (!lsx_compandt_parse(&l->transfer_fn, argv[1], n>2 ? argv[2] : 0))
+  if (!lsx_compandt_parse(&l->transfer_fn, argv[1], argc>2 ? argv[2] : 0))
     return SOX_EOF;
 
   /* Set the initial "volume" to be attibuted to the input channels.
@@ -103,7 +104,7 @@ static int getopts(sox_effect_t * effp, int n, char * * argv)
      result if the user has seleced a long attack time */
   for (i = 0; i < l->expectedChannels; ++i) {
     double init_vol_dB = 0;
-    if (n > 3 && sscanf(argv[3], "%lf %c", &init_vol_dB, &dummy) != 1) {
+    if (argc > 3 && sscanf(argv[3], "%lf %c", &init_vol_dB, &dummy) != 1) {
       lsx_fail("syntax error trying to read initial volume");
       return SOX_EOF;
     } else if (init_vol_dB > 0) {
@@ -114,7 +115,7 @@ static int getopts(sox_effect_t * effp, int n, char * * argv)
   }
 
   /* If there is a delay, store it. */
-  if (n > 4 && sscanf(argv[4], "%lf %c", &l->delay, &dummy) != 1) {
+  if (argc > 4 && sscanf(argv[4], "%lf %c", &l->delay, &dummy) != 1) {
     lsx_fail("syntax error trying to read delay value");
     return SOX_EOF;
   } else if (l->delay < 0) {
@@ -241,7 +242,7 @@ static int drain(sox_effect_t * effp, sox_sample_t *obuf, size_t *osamp)
 
   if (l->delay_buf_full == 0)
     l->delay_buf_index = 0;
-  while (done < *osamp && l->delay_buf_cnt > 0)
+  while (done+effp->out_signal.channels <= *osamp && l->delay_buf_cnt > 0)
     for (chan = 0; chan < effp->out_signal.channels; ++chan) {
       int c = l->expectedChannels > 1 ? chan : 0;
       double level_in_lin = l->channels[c].volume;
@@ -262,7 +263,7 @@ static int stop(sox_effect_t * effp)
   return SOX_SUCCESS;
 }
 
-static int kill(sox_effect_t * effp)
+static int lsx_kill(sox_effect_t * effp)
 {
   priv_t * l = (priv_t *) effp->priv;
 
@@ -271,11 +272,11 @@ static int kill(sox_effect_t * effp)
   return SOX_SUCCESS;
 }
 
-sox_effect_handler_t const * sox_compand_effect_fn(void)
+sox_effect_handler_t const * lsx_compand_effect_fn(void)
 {
   static sox_effect_handler_t handler = {
-    "compand", compand_usage, SOX_EFF_MCHAN,
-    getopts, start, flow, drain, stop, kill, sizeof(priv_t)
+    "compand", compand_usage, SOX_EFF_MCHAN | SOX_EFF_GAIN,
+    getopts, start, flow, drain, stop, lsx_kill, sizeof(priv_t)
   };
   return &handler;
 }
