@@ -1,6 +1,6 @@
 /* libSoX ffmpeg formats.
  *
- * Copyright 2007 Reuben Thomas <rrt@sc3d.org>
+ * Copyright 2007, 2011 Reuben Thomas <rrt@sc3d.org>
  *
  * Based on ffplay.c and output_example.c Copyright 2003 Fabrice Bellard
  * Note: ffplay.c is distributed under the LGPL 2.1 or later;
@@ -50,12 +50,19 @@
 #include <ctype.h>
 #include "ffmpeg.h"
 
+#ifndef CODEC_TYPE_AUDIO
+#define CODEC_TYPE_AUDIO AVMEDIA_TYPE_AUDIO
+#endif
+#ifndef PKT_FLAG_KEY
+#define PKT_FLAG_KEY AV_PKT_FLAG_KEY
+#endif
+
 /* Private data for ffmpeg files */
 typedef struct {
   int audio_index;
   int audio_stream;
   AVStream *audio_st;
-  uint8_t *audio_buf;
+  uint8_t *audio_buf_aligned;
   int audio_buf_index, audio_buf_size;
   int16_t *samples;
   int samples_index;
@@ -63,8 +70,7 @@ typedef struct {
   AVFormatContext *ctxt;
   int audio_input_frame_size;
   AVPacket audio_pkt;
-  uint8_t *audio_pkt_data;
-  int audio_pkt_size;
+  uint8_t *audio_buf_raw;
 } priv_t;
 
 /* open a given stream. Return 0 if OK */
@@ -92,7 +98,7 @@ static int stream_component_open(priv_t * ffmpeg, int stream_index)
 
   if (!codec || avcodec_open(enc, codec) < 0)
     return -1;
-  if (enc->codec_type != CODEC_TYPE_AUDIO) {
+  if (enc->codec_type != AVMEDIA_TYPE_AUDIO) {
     lsx_fail("ffmpeg CODEC %x is not an audio CODEC", enc->codec_type);
     return -1;
   }
@@ -127,26 +133,27 @@ static int audio_decode_frame(priv_t * ffmpeg, uint8_t *audio_buf, int buf_size)
 
   for (;;) {
     /* NOTE: the audio packet can contain several frames */
-    while (ffmpeg->audio_pkt_size > 0) {
+    while (ffmpeg->audio_pkt.size > 0) {
       data_size = buf_size;
-      len1 = avcodec_decode_audio2(ffmpeg->audio_st->codec,
-                                   (int16_t *)audio_buf, &data_size,
-                                   ffmpeg->audio_pkt_data, ffmpeg->audio_pkt_size);
+      len1 = avcodec_decode_audio3(ffmpeg->audio_st->codec,
+				   (int16_t *)audio_buf, &data_size,
+				   pkt);
       if (len1 < 0) /* if error, we skip the rest of the packet */
 	return 0;
 
-      ffmpeg->audio_pkt_data += len1;
-      ffmpeg->audio_pkt_size -= len1;
+      ffmpeg->audio_pkt.data += len1;
+      ffmpeg->audio_pkt.size -= len1;
       if (data_size <= 0)
-        continue;
+	continue;
       return data_size;
     }
-
-    ffmpeg->audio_pkt_data = pkt->data;
-    ffmpeg->audio_pkt_size = pkt->size;
   }
 }
 
+/* On some platforms, libavcodec wants the output buffer aligned to 16
+ * bytes (because it uses SSE/Altivec internally). */
+#define ALIGN16(p) ((uint8_t *)(p) + (16 - (size_t)(p) % 16))
+
 static int startread(sox_format_t * ft)
 {
   priv_t * ffmpeg = (priv_t *)ft->priv;
@@ -154,7 +161,8 @@ static int startread(sox_format_t * ft)
   int ret;
   int i;
 
-  ffmpeg->audio_buf = lsx_calloc(1, (size_t)AVCODEC_MAX_AUDIO_FRAME_SIZE);
+  ffmpeg->audio_buf_raw = lsx_calloc(1, (size_t)AVCODEC_MAX_AUDIO_FRAME_SIZE + 32);
+  ffmpeg->audio_buf_aligned = ALIGN16(ffmpeg->audio_buf_raw);
 
   /* Signal audio stream not found */
   ffmpeg->audio_index = -1;
@@ -181,7 +189,7 @@ static int startread(sox_format_t * ft)
   /* Find audio stream (FIXME: allow different stream to be selected) */
   for (i = 0; (unsigned)i < ffmpeg->ctxt->nb_streams; i++) {
     AVCodecContext *enc = ffmpeg->ctxt->streams[i]->codec;
-    if (enc->codec_type == CODEC_TYPE_AUDIO && ffmpeg->audio_index < 0) {
+    if (enc->codec_type == AVMEDIA_TYPE_AUDIO && ffmpeg->audio_index < 0) {
       ffmpeg->audio_index = i;
       break;
     }
@@ -201,8 +209,8 @@ static int startread(sox_format_t * ft)
   ft->encoding.encoding = SOX_ENCODING_SIGN2;
   ft->signal.channels = ffmpeg->audio_st->codec->channels;
   ft->signal.length = 0; /* Currently we can't seek; no idea how to get this
-                     info from ffmpeg anyway (in time, yes, but not in
-                     samples); but ffmpeg *can* seek */
+		     info from ffmpeg anyway (in time, yes, but not in
+		     samples); but ffmpeg *can* seek */
 
   return SOX_SUCCESS;
 }
@@ -224,15 +232,15 @@ static size_t read_samples(sox_format_t * ft, sox_sample_t *buf, size_t len)
     if (ffmpeg->audio_buf_index * 2 >= ffmpeg->audio_buf_size) {
       if ((ret = av_read_frame(ffmpeg->ctxt, pkt)) < 0 &&
 	  (ret == AVERROR_EOF || url_ferror(ffmpeg->ctxt->pb)))
-        break;
-      ffmpeg->audio_buf_size = audio_decode_frame(ffmpeg, ffmpeg->audio_buf, AVCODEC_MAX_AUDIO_FRAME_SIZE);
+	break;
+      ffmpeg->audio_buf_size = audio_decode_frame(ffmpeg, ffmpeg->audio_buf_aligned, AVCODEC_MAX_AUDIO_FRAME_SIZE);
       ffmpeg->audio_buf_index = 0;
     }
 
     /* Convert data into SoX samples up to size of buffer */
     nextra = min((ffmpeg->audio_buf_size - ffmpeg->audio_buf_index) / 2, (int)(len - nsamp));
     for (; nextra > 0; nextra--)
-      buf[nsamp++] = SOX_SIGNED_16BIT_TO_SAMPLE(((int16_t *)ffmpeg->audio_buf)[ffmpeg->audio_buf_index++], ft->clips);
+      buf[nsamp++] = SOX_SIGNED_16BIT_TO_SAMPLE(((int16_t *)ffmpeg->audio_buf_aligned)[ffmpeg->audio_buf_index++], ft->clips);
   } while (nsamp < len && nextra > 0);
 
   return nsamp;
@@ -252,6 +260,7 @@ static int stopread(sox_format_t * ft)
     ffmpeg->ctxt = NULL; /* safety */
   }
 
+  free(ffmpeg->audio_buf_raw);
   return SOX_SUCCESS;
 }
 
@@ -271,7 +280,7 @@ static AVStream *add_audio_stream(sox_format_t * ft, AVFormatContext *oc, enum C
 
   c = st->codec;
   c->codec_id = codec_id;
-  c->codec_type = CODEC_TYPE_AUDIO;
+  c->codec_type = AVMEDIA_TYPE_AUDIO;
 
   /* put sample parameters */
   c->bit_rate = 256000;  /* FIXME: allow specification */
@@ -302,7 +311,8 @@ static int open_audio(priv_t * ffmpeg, AVStream *st)
     return SOX_EOF;
   }
 
-  ffmpeg->audio_buf = lsx_malloc((size_t)AVCODEC_MAX_AUDIO_FRAME_SIZE);
+  ffmpeg->audio_buf_raw = lsx_malloc((size_t)AVCODEC_MAX_AUDIO_FRAME_SIZE + 32);
+  ffmpeg->audio_buf_aligned = ALIGN16(ffmpeg->audio_buf_raw);
 
   /* ugly hack for PCM codecs (will be removed ASAP with new PCM
      support to compute the input frame size in samples */
@@ -335,10 +345,10 @@ static int startwrite(sox_format_t * ft)
 
   /* auto detect the output format from the name. default is
      mpeg. */
-  ffmpeg->fmt = guess_format(NULL, ft->filename, NULL);
+  ffmpeg->fmt = av_guess_format(NULL, ft->filename, NULL);
   if (!ffmpeg->fmt) {
     lsx_warn("ffmpeg could not deduce output format from file extension; using MPEG");
-    ffmpeg->fmt = guess_format("mpeg", NULL, NULL);
+    ffmpeg->fmt = av_guess_format("mpeg", NULL, NULL);
     if (!ffmpeg->fmt) {
       lsx_fail("ffmpeg could not find suitable output format");
       return SOX_EOF;
@@ -346,7 +356,7 @@ static int startwrite(sox_format_t * ft)
   }
 
   /* allocate the output media context */
-  ffmpeg->ctxt = av_alloc_format_context();
+  ffmpeg->ctxt = avformat_alloc_context();
   if (!ffmpeg->ctxt) {
     fprintf(stderr, "ffmpeg out of memory error");
     return SOX_EOF;
@@ -408,28 +418,28 @@ static size_t write_samples(sox_format_t * ft, const sox_sample_t *buf, size_t l
     if (ffmpeg->samples_index < ffmpeg->audio_input_frame_size) {
       SOX_SAMPLE_LOCALS;
       for (; nread < len && ffmpeg->samples_index < ffmpeg->audio_input_frame_size; nread++)
-        ffmpeg->samples[ffmpeg->samples_index++] = SOX_SAMPLE_TO_SIGNED_16BIT(buf[nread], ft->clips);
+	ffmpeg->samples[ffmpeg->samples_index++] = SOX_SAMPLE_TO_SIGNED_16BIT(buf[nread], ft->clips);
     }
 
     /* If output frame full or no more data to read, write it out */
     if (ffmpeg->samples_index == ffmpeg->audio_input_frame_size ||
-        (len == 0 && ffmpeg->samples_index > 0)) {
+	(len == 0 && ffmpeg->samples_index > 0)) {
       AVCodecContext *c = ffmpeg->audio_st->codec;
       AVPacket pkt;
 
       av_init_packet(&pkt);
-      pkt.size = avcodec_encode_audio(c, ffmpeg->audio_buf, AVCODEC_MAX_AUDIO_FRAME_SIZE, ffmpeg->samples);
+      pkt.size = avcodec_encode_audio(c, ffmpeg->audio_buf_aligned, AVCODEC_MAX_AUDIO_FRAME_SIZE, ffmpeg->samples);
       pkt.pts = av_rescale_q(c->coded_frame->pts, c->time_base, ffmpeg->audio_st->time_base);
-      pkt.flags |= PKT_FLAG_KEY;
+      pkt.flags |= AV_PKT_FLAG_KEY;
       pkt.stream_index = ffmpeg->audio_st->index;
-      pkt.data = ffmpeg->audio_buf;
+      pkt.data = ffmpeg->audio_buf_aligned;
 
       /* write the compressed frame to the media file */
       if (av_write_frame(ffmpeg->ctxt, &pkt) != 0)
-        lsx_fail("ffmpeg had error while writing audio frame");
+	lsx_fail("ffmpeg had error while writing audio frame");
 
       /* Increment nwritten whether write succeeded or not; we have to
-         get rid of the input! */
+	 get rid of the input! */
       nwritten += ffmpeg->samples_index;
       ffmpeg->samples_index = 0;
     }
@@ -449,10 +459,11 @@ static int stopwrite(sox_format_t * ft)
   /* Close CODEC */
   if (ffmpeg->audio_st) {
     avcodec_close(ffmpeg->audio_st->codec);
-    free(ffmpeg->samples);
-    free(ffmpeg->audio_buf);
   }
 
+  free(ffmpeg->samples);
+  free(ffmpeg->audio_buf_raw);
+
   /* Write the trailer, if any */
   av_write_trailer(ffmpeg->ctxt);
 
@@ -485,6 +496,7 @@ LSX_FORMAT_HANDLER(ffmpeg)
     "ffmpeg", /* special type to force use of ffmpeg */
     "mp4",
     "m4a",
+    "m4b",
     "avi",
     "wmv",
     "mpg",
