@@ -163,7 +163,7 @@ static int parse_subarg(char *s, char **subargv, size_t *subargc) {
 static int getopts(sox_effect_t * effp, int argc, char **argv)
 {
   char *subargv[6], *cp;
-  size_t subargc, i, len;
+  size_t subargc, i;
 
   priv_t * c = (priv_t *) effp->priv;
   --argc, ++argv;
@@ -182,7 +182,6 @@ static int getopts(sox_effect_t * effp, int argc, char **argv)
   c->bands = lsx_calloc(c->nBands, sizeof(comp_band_t));
 
   for (i=0;i<c->nBands;++i) {
-    len = strlen(argv[i<<1]);
     if (parse_subarg(argv[i<<1],subargv,&subargc) != SOX_SUCCESS)
       return SOX_EOF;
     if (sox_mcompand_getopts_1(&c->bands[i], subargc, &subargv[0]) != SOX_SUCCESS)
@@ -270,9 +269,10 @@ static void doVolume(double *v, double samp, comp_band_t * l, size_t chan)
 
 static int sox_mcompand_flow_1(sox_effect_t * effp, priv_t * c, comp_band_t * l, const sox_sample_t *ibuf, sox_sample_t *obuf, size_t len, size_t filechans)
 {
-  size_t done, chan;
+  size_t idone, odone;
 
-  for (done = 0; done < len; ibuf += filechans) {
+  for (idone = 0, odone = 0; idone < len; ibuf += filechans) {
+    size_t chan;
 
     /* Maintain the volume fields by simulating a leaky pump circuit */
 
@@ -300,8 +300,8 @@ static int sox_mcompand_flow_1(sox_effect_t * effp, priv_t * c, comp_band_t * l,
       if (c->delay_buf_size <= 0) {
         checkbuf = ibuf[chan] * level_out_lin;
         SOX_SAMPLE_CLIP_COUNT(checkbuf, effp->clips);
-        obuf[done++] = checkbuf;
-
+        obuf[odone++] = checkbuf;
+        idone++;
       } else {
         /* FIXME: note that this lookahead algorithm is really lame:
            the response to a peak is released before the peak
@@ -321,16 +321,30 @@ static int sox_mcompand_flow_1(sox_effect_t * effp, priv_t * c, comp_band_t * l,
           SOX_SAMPLE_CLIP_COUNT(checkbuf, effp->clips);
           l->delay_buf[(l->delay_buf_ptr + c->delay_buf_size - l->delay_size)%c->delay_buf_size] = checkbuf;
         }
-        if (l->delay_buf_cnt >= c->delay_buf_size)
-          obuf[done++] = l->delay_buf[l->delay_buf_ptr];
-        else
+        if (l->delay_buf_cnt >= c->delay_buf_size) {
+          obuf[odone] = l->delay_buf[l->delay_buf_ptr];
+          odone++;
+          idone++;
+        } else {
           l->delay_buf_cnt++;
+          idone++; /* no "odone++" because we did not fill obuf[...] */
+        }
         l->delay_buf[l->delay_buf_ptr++] = ibuf[chan];
         l->delay_buf_ptr %= c->delay_buf_size;
       }
     }
   }
 
+  if (idone != odone || idone != len) {
+    /* Emergency brake - will lead to memory corruption otherwise since we
+       cannot report back to flow() how many samples were consumed/emitted.
+       Additionally, flow() doesn't know how to handle diverging
+       sub-compander delays. */
+    lsx_fail("Using a compander delay within mcompand is currently not supported");
+    exit(1);
+    /* FIXME */
+  }
+
   return (SOX_SUCCESS);
 }
 
@@ -354,6 +368,8 @@ static int flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obu
     c->band_buf_len = len;
   }
 
+  len -= len % effp->out_signal.channels;
+
   ibuf_copy = lsx_malloc(*isamp * sizeof(sox_sample_t));
   memcpy(ibuf_copy, ibuf, *isamp * sizeof(sox_sample_t));
 
@@ -420,6 +436,8 @@ static int drain(sox_effect_t * effp, sox_sample_t *obuf, size_t *osamp)
   priv_t * c = (priv_t *)effp->priv;
   comp_band_t * l;
 
+  *osamp -= *osamp % effp->out_signal.channels;
+
   memset(obuf,0,*osamp * sizeof *obuf);
   for (band=0;band<c->nBands;++band) {
     l = &c->bands[band];
@@ -462,7 +480,7 @@ static int stop(sox_effect_t * effp)
   return SOX_SUCCESS;
 }
 
-static int kill(sox_effect_t * effp)
+static int lsx_kill(sox_effect_t * effp)
 {
   priv_t * c = (priv_t *) effp->priv;
   comp_band_t * l;
@@ -493,7 +511,7 @@ const sox_effect_handler_t *lsx_mcompand_effect_fn(void)
     "                 in-dB1,out-dB1[,in-dB2,out-dB2...]\n"
     "                [ gain [ initial-volume [ delay ] ] ]",
     SOX_EFF_MCHAN | SOX_EFF_GAIN,
-    getopts, start, flow, drain, stop, kill, sizeof(priv_t)
+    getopts, start, flow, drain, stop, lsx_kill, sizeof(priv_t)
   };
 
   return &handler;
