@@ -1,4 +1,4 @@
-/* noiseprof - Noise Profiling Effect.
+/* noisered - Noise Reduction Effect.
  *
  * Written by Ian Turner (vectro@vectro.org)
  *
@@ -60,6 +60,7 @@ static void FFT(unsigned NumSamples,
 static int sox_noisered_getopts(sox_effect_t * effp, int argc, char **argv)
 {
   priv_t * p = (priv_t *) effp->priv;
+  --argc, ++argv;
 
   if (argc > 0) {
     p->profile_filename = argv[0];
@@ -85,7 +86,10 @@ static int sox_noisered_start(sox_effect_t * effp)
     size_t fchannels = 0;
     size_t channels = effp->in_signal.channels;
     size_t i;
-    FILE* ifp;
+    FILE * ifp = lsx_open_input_file(effp, data->profile_filename);
+
+    if (!ifp)
+      return SOX_EOF;
 
     data->chandata = lsx_calloc(channels, sizeof(*(data->chandata)));
     data->bufdata = 0;
@@ -94,22 +98,6 @@ static int sox_noisered_start(sox_effect_t * effp)
         data->chandata[i].smoothing = lsx_calloc(FREQCOUNT, sizeof(float));
         data->chandata[i].lastwindow = NULL;
     }
-
-    /* Here we actually open the input file. */
-    if (!data->profile_filename || !strcmp(data->profile_filename, "-")) {
-      if (effp->global_info->global_info->stdin_in_use_by) {
-        lsx_fail("stdin already in use by '%s'", effp->global_info->global_info->stdin_in_use_by);
-        return SOX_EOF;
-      }
-      effp->global_info->global_info->stdin_in_use_by = effp->handler.name;
-      ifp = stdin;
-    }
-    else if ((ifp = fopen(data->profile_filename, "r")) == NULL) {
-        lsx_fail("Couldn't open profile file %s: %s",
-                data->profile_filename, strerror(errno));
-        return SOX_EOF;
-    }
-
     while (1) {
         unsigned long i1_ul;
         size_t i1;
@@ -126,7 +114,7 @@ static int sox_noisered_start(sox_effect_t * effp)
         data->chandata[fchannels].noisegate[0] = f1;
         for (i = 1; i < FREQCOUNT; i ++) {
             if (1 != fscanf(ifp, ", %f", &f1)) {
-                lsx_fail("noisered: Not enough datums for channel %lu "
+                lsx_fail("noisered: Not enough data for channel %lu "
                         "(expected %d, got %lu)", (unsigned long)fchannels, FREQCOUNT, (unsigned long)i);
                 return SOX_EOF;
             }
@@ -142,6 +130,8 @@ static int sox_noisered_start(sox_effect_t * effp)
     if (ifp != stdin)
       fclose(ifp);
 
+  effp->out_signal.length = SOX_UNKNOWN_LEN; /* TODO: calculate actual length */
+
     return (SOX_SUCCESS);
 }
 
@@ -230,6 +220,7 @@ static int process_window(sox_effect_t * effp, priv_t * data, unsigned chan_num,
     int use = min(len, WINDOWSIZE)-min(len,(WINDOWSIZE/2));
     chandata_t *chan = &(data->chandata[chan_num]);
     int first = (chan->lastwindow == NULL);
+    SOX_SAMPLE_LOCALS;
 
     if ((nextwindow = lsx_calloc(WINDOWSIZE, sizeof(float))) == NULL)
         return SOX_EOF;
@@ -283,6 +274,7 @@ static int sox_noisered_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_
 
     /* Reduce noise on every channel. */
     for (i = 0; i < tracks; i ++) {
+        SOX_SAMPLE_LOCALS;
         chandata_t* chan = &(data->chandata[i]);
         size_t j;
 
@@ -359,7 +351,7 @@ static sox_effect_handler_t sox_noisered_effect = {
   NULL, sizeof(priv_t)
 };
 
-const sox_effect_handler_t *sox_noisered_effect_fn(void)
+const sox_effect_handler_t *lsx_noisered_effect_fn(void)
 {
     return &sox_noisered_effect;
 }
