@@ -25,7 +25,7 @@ typedef struct {
   double dsum1, dsum2;          /* deltas */
   double scale;                 /* scale-factor */
   double last;                  /* previous sample */
-  size_t read;               /* samples processed */
+  uint64_t read;               /* samples processed */
   int volume;
   int srms;
   int fft;
@@ -40,7 +40,7 @@ typedef struct {
 /*
  * Process options
  */
-static int sox_stat_getopts(sox_effect_t * effp, int n, char **argv)
+static int sox_stat_getopts(sox_effect_t * effp, int argc, char **argv)
 {
   priv_t * stat = (priv_t *) effp->priv;
 
@@ -49,15 +49,16 @@ static int sox_stat_getopts(sox_effect_t * effp, int n, char **argv)
   stat->srms = 0;
   stat->fft = 0;
 
-  for (; n > 0; n--, argv++) {
+  --argc, ++argv;
+  for (; argc > 0; argc--, argv++) {
     if (!(strcmp(*argv, "-v")))
       stat->volume = 1;
     else if (!(strcmp(*argv, "-s"))) {
-      if (n <= 1) {
+      if (argc <= 1) {
         lsx_fail("-s option: invalid argument");
         return SOX_EOF;
       }
-      n--, argv++;              /* Move to next argument. */
+      argc--, argv++;              /* Move to next argument. */
       if (!sscanf(*argv, "%lf", &stat->scale)) {
         lsx_fail("-s option: invalid argument");
         return SOX_EOF;
@@ -140,6 +141,7 @@ static int sox_stat_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_samp
 
     if (stat->fft) {
       for (x = 0; x < len; x++) {
+        SOX_SAMPLE_LOCALS;
         stat->re_in[stat->fft_offset++] = SOX_SAMPLE_TO_FLOAT_32BIT(ibuf[x], effp->clips);
 
         if (stat->fft_offset >= stat->fft_size) {
@@ -263,7 +265,7 @@ static int sox_stat_stop(sox_effect_t * effp)
   if (stat->volume == 2)
     fprintf(stderr, "\n\n");
   /* print out the info */
-  fprintf(stderr, "Samples read:      %12lu\n", (unsigned long)stat->read);
+  fprintf(stderr, "Samples read:      %12" PRIu64 "\n", stat->read);
   fprintf(stderr, "Length (seconds):  %12.6f\n", (double)stat->read/effp->in_signal.rate/effp->in_signal.channels);
   if (stat->srms)
     fprintf(stderr, "Scaled by rms:     %12.6f\n", rms);
@@ -319,7 +321,7 @@ static int sox_stat_stop(sox_effect_t * effp)
 static sox_effect_handler_t sox_stat_effect = {
   "stat",
   "[ -s N ] [ -rms ] [-freq] [ -v ] [ -d ]",
-  SOX_EFF_MCHAN,
+  SOX_EFF_MCHAN | SOX_EFF_MODIFY,
   sox_stat_getopts,
   sox_stat_start,
   sox_stat_flow,
@@ -328,7 +330,7 @@ static sox_effect_handler_t sox_stat_effect = {
   NULL, sizeof(priv_t)
 };
 
-const sox_effect_handler_t *sox_stat_effect_fn(void)
+const sox_effect_handler_t *lsx_stat_effect_fn(void)
 {
   return &sox_stat_effect;
 }
